@@ -19,6 +19,9 @@
 
 #include "gtkcssnodedeclarationprivate.h"
 
+/* XXX: For g_memdup2() */
+#include "gtkprivate.h"
+
 #include <string.h>
 
 struct _GtkCssNodeDeclaration {
@@ -51,7 +54,7 @@ gtk_css_node_declaration_make_writable (GtkCssNodeDeclaration **decl)
 
   (*decl)->refcount--;
 
-  *decl = g_memdup (*decl, sizeof_this_node (*decl));
+  *decl = g_memdup2 (*decl, sizeof_this_node (*decl));
   (*decl)->refcount = 1;
 }
 
@@ -180,7 +183,7 @@ find_class (const GtkCssNodeDeclaration *decl,
             GQuark                       class_quark,
             guint                       *position)
 {
-  gint min, max, mid;
+  int min, max, mid;
   gboolean found = FALSE;
   guint pos;
 
