@@ -47,7 +47,7 @@ typedef struct _GtkIconTheme      GtkIconTheme;
  * @GTK_ICON_LOOKUP_PRELOAD: Starts loading the texture in the background
  *   so it is ready when later needed.
  *
- * Used to specify options for gtk_icon_theme_lookup_icon()
+ * Used to specify options for gtk_icon_theme_lookup_icon().
  */
 typedef enum
 {
@@ -68,7 +68,7 @@ typedef enum
  * @GTK_ICON_THEME_NOT_FOUND: The icon specified does not exist in the theme
  * @GTK_ICON_THEME_FAILED: An unspecified error occurred.
  *
- * Error codes for GtkIconTheme operations.
+ * Error codes for `GtkIconTheme` operations.
  **/
 typedef enum {
   GTK_ICON_THEME_NOT_FOUND,
@@ -86,6 +86,9 @@ GtkIconTheme    *gtk_icon_theme_new                  (void);
 GDK_AVAILABLE_IN_ALL
 GtkIconTheme    *gtk_icon_theme_get_for_display      (GdkDisplay                  *display);
 
+GDK_AVAILABLE_IN_ALL
+GdkDisplay *     gtk_icon_theme_get_display          (GtkIconTheme                *self);
+
 GDK_AVAILABLE_IN_ALL
 void             gtk_icon_theme_set_search_path      (GtkIconTheme                *self,
                                                       const char * const          *path);
