@@ -48,9 +48,7 @@
 #include "gtkaccessibleprivate.h"
 
 /*< private >
- * SECTION:gtkmodelbutton
- * @Short_description: A button that uses a GAction as model
- * @Title: GtkModelButton
+ * GtkModelButton:
  *
  * GtkModelButton is a button class that can use a #GAction as its model.
  * In contrast to #GtkToggleButton or #GtkCheckButton, which can also
@@ -73,7 +71,7 @@
  * and #GtkModelButton:icon properties.
  *
  * The appearance of model buttons can be influenced with the
- * #GtkModelButton:centered and #GtkModelButton:iconic properties.
+ * #GtkModelButton:iconic property.
  *
  * Model buttons have built-in support for submenus in #GtkPopoverMenu.
  * To make a GtkModelButton that opens a submenu when activated, set
@@ -177,7 +175,6 @@ struct _GtkModelButton
   GtkEventController *controller;
 
   guint active : 1;
-  guint centered : 1;
   guint iconic : 1;
   guint keep_open : 1;
 };
@@ -490,7 +487,7 @@ update_node_name (GtkModelButton *self)
     {
     case GTK_BUTTON_ROLE_TITLE:
       start_name = "arrow";
-      end_name = NULL;
+      end_name = "";
       break;
     case GTK_BUTTON_ROLE_NORMAL:
       start_name = NULL;
@@ -760,8 +757,6 @@ gtk_model_button_set_iconic (GtkModelButton *self,
       gtk_widget_add_css_class (widget, "flat");
     }
 
-  self->centered = iconic;
-
   if (!iconic)
     {
       if (self->start_indicator)
