@@ -19,31 +19,28 @@
 
 
 /**
- * SECTION:gtkshortcutcontroller
- * @Short_description: Event controller for shortcuts
- * @Title: GtkShortcutController
- * @See_also: #GtkEventController, #GtkShortcut
+ * GtkShortcutController:
  *
- * #GtkShortcutController is an event controller that manages shortcuts.
+ * `GtkShortcutController` is an event controller that manages shortcuts.
  *
  * Most common shortcuts are using this controller implicitly, e.g. by
- * adding a mnemonic underline to a #GtkLabel, or by installing a key
+ * adding a mnemonic underline to a `GtkLabel`, or by installing a key
  * binding using gtk_widget_class_add_binding(), or by adding accelerators
  * to global actions using gtk_application_set_accels_for_action().
  *
  * But it is possible to create your own shortcut controller, and add
  * shortcuts to it.
  *
- * #GtkShortcutController implements #GListModel for querying the shortcuts that
- * have been added to it.
+ * `GtkShortcutController` implements `GListModel` for querying the
+ * shortcuts that have been added to it.
  *
  * # GtkShortcutController as a GtkBuildable
  *
- * GtkShortcutControllers can be creates in ui files to set up shortcuts
- * in the same place as the widgets.
+ * `GtkShortcutControllers` can be creates in ui files to set up
+ * shortcuts in the same place as the widgets.
  *
- * An example of a UI definition fragment with GtkShortcutController:
- * |[
+ * An example of a UI definition fragment with `GtkShortcutController`:
+ * ```xml
  *   <object class='GtkButton'>
  *     <child>
  *       <object class='GtkShortcutController'>
@@ -57,13 +54,14 @@
  *       </object>
  *     </child>
  *   </object>
- * ]|
+ * ```
  *
- * This example creates a #GtkActivateAction for triggering the `activate`
- * signal of the GtkButton. See gtk_shortcut_action_parse_string() for the syntax
- * for other kinds of #GtkShortcutAction. See gtk_shortcut_trigger_parse_string()
- * to learn more about the syntax for triggers.
- **/
+ * This example creates a [class@Gtk.ActivateAction] for triggering the
+ * `activate` signal of the `GtkButton`. See [ctor@Gtk.ShortcutAction.parse_string]
+ * for the syntax for other kinds of `GtkShortcutAction`. See
+ * [ctor@Gtk.ShortcutTrigger.parse_string] to learn more about the syntax
+ * for triggers.
+ */
 
 #include "config.h"
 
@@ -91,6 +89,7 @@ struct _GtkShortcutController
   GtkShortcutScope scope;
   GdkModifierType mnemonics_modifiers;
 
+  gulong shortcuts_changed_id;
   guint custom_shortcuts : 1;
 
   guint last_activated;
@@ -213,7 +212,11 @@ gtk_shortcut_controller_set_property (GObject      *object,
             self->shortcuts = g_object_ref (model);
             self->custom_shortcuts = FALSE;
           }
-        g_signal_connect_swapped (self->shortcuts, "items-changed", G_CALLBACK (g_list_model_items_changed), self);
+
+        self->shortcuts_changed_id =  g_signal_connect_swapped (self->shortcuts,
+                                                                "items-changed",
+                                                                G_CALLBACK (g_list_model_items_changed),
+                                                                self);
       }
       break;
 
@@ -265,7 +268,7 @@ gtk_shortcut_controller_finalize (GObject *object)
 {
   GtkShortcutController *self = GTK_SHORTCUT_CONTROLLER (object);
 
-  g_signal_handlers_disconnect_by_func (self->shortcuts, g_list_model_items_changed, self);
+  g_clear_signal_handler (&self->shortcuts_changed_id, self->shortcuts);
   g_clear_object (&self->shortcuts);
 
   G_OBJECT_CLASS (gtk_shortcut_controller_parent_class)->finalize (object);
@@ -442,27 +445,26 @@ gtk_shortcut_controller_handle_event (GtkEventController *controller,
 }
 
 static void
-update_accel (GtkShortcut *shortcut,
-              GtkWidget   *widget,
-              gboolean     set)
+update_accel (GtkShortcut    *shortcut,
+              GtkActionMuxer *muxer,
+              gboolean        set)
 {
   GtkShortcutTrigger *trigger;
   GtkShortcutAction *action;
-  GtkActionMuxer *muxer;
   GVariant *target;
   const char *action_name;
   char *action_and_target;
   char *accel = NULL;
 
-  trigger = gtk_shortcut_get_trigger (shortcut);
-  action = gtk_shortcut_get_action (shortcut);
-
-  if (!GTK_IS_NAMED_ACTION (action) ||
-      !GTK_IS_KEYVAL_TRIGGER (trigger))
+  if (!muxer)
     return;
 
-  muxer = _gtk_widget_get_action_muxer (widget, set);
-  if (!muxer)
+  action = gtk_shortcut_get_action (shortcut);
+  if (!GTK_IS_NAMED_ACTION (action))
+    return;
+
+  trigger = gtk_shortcut_get_trigger (shortcut);
+  if (!GTK_IS_KEYVAL_TRIGGER (trigger))
     return;
 
   target = gtk_shortcut_get_arguments (shortcut);
@@ -481,15 +483,18 @@ gtk_shortcut_controller_set_widget (GtkEventController *controller,
                                     GtkWidget          *widget)
 {
   GtkShortcutController *self = GTK_SHORTCUT_CONTROLLER (controller);
+  GListModel *shortcuts = self->shortcuts;
+  GtkActionMuxer *muxer;
   guint i, p;
 
   GTK_EVENT_CONTROLLER_CLASS (gtk_shortcut_controller_parent_class)->set_widget (controller, widget);
 
-  for (i = 0, p = g_list_model_get_n_items (G_LIST_MODEL (controller)); i < p; i++)
+  muxer = _gtk_widget_get_action_muxer (widget, TRUE);
+  for (i = 0, p = g_list_model_get_n_items (shortcuts); i < p; i++)
     {
-      GtkShortcut *shortcut = g_list_model_get_item (G_LIST_MODEL (controller), i);
+      GtkShortcut *shortcut = g_list_model_get_item (shortcuts, i);
       if (GTK_IS_SHORTCUT (shortcut))
-        update_accel (shortcut, widget, TRUE);
+        update_accel (shortcut, muxer, TRUE);
       g_object_unref (shortcut);
     }
 
@@ -536,7 +541,7 @@ gtk_shortcut_controller_class_init (GtkShortcutControllerClass *klass)
   controller_class->unset_widget = gtk_shortcut_controller_unset_widget;
 
   /**
-   * GtkShortcutController:mnemonic-modifiers:
+   * GtkShortcutController:mnemonic-modifiers: (attributes org.gtk.Property.get=gtk_shortcut_controller_get_mnemonics_modifiers org.gtk.Property.set=gtk_shortcut_controller_set_mnemonics_modifiers)
    *
    * The modifiers that need to be pressed to allow mnemonics activation.
    */
@@ -551,7 +556,7 @@ gtk_shortcut_controller_class_init (GtkShortcutControllerClass *klass)
   /**
    * GtkShortcutController:model:
    *
-   * A list model to take shortcuts from
+   * A list model to take shortcuts from.
    */
   properties[PROP_MODEL] =
       g_param_spec_object ("model",
@@ -561,7 +566,7 @@ gtk_shortcut_controller_class_init (GtkShortcutControllerClass *klass)
                            G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GtkShortcutController:scope:
+   * GtkShortcutController:scope: (attributes org.gtk.Property.get=gtk_shortcut_controller_get_scope org.gtk.Property.set=gtk_shortcut_controller_set_scope)
    *
    * What scope the shortcuts will be handled in.
    */
@@ -688,7 +693,7 @@ gtk_shortcut_controller_new (void)
 
 /**
  * gtk_shortcut_controller_new_for_model:
- * @model: a #GListModel containing shortcuts
+ * @model: a `GListModel` containing shortcuts
  *
  * Creates a new shortcut controller that takes its shortcuts from
  * the given list model.
@@ -712,13 +717,13 @@ gtk_shortcut_controller_new_for_model (GListModel *model)
 /**
  * gtk_shortcut_controller_add_shortcut:
  * @self: the controller
- * @shortcut: (transfer full): a #GtkShortcut
+ * @shortcut: (transfer full): a `GtkShortcut`
  *
  * Adds @shortcut to the list of shortcuts handled by @self.
  *
  * If this controller uses an external shortcut list, this
  * function does nothing.
- **/
+ */
 void
 gtk_shortcut_controller_add_shortcut (GtkShortcutController *self,
                                       GtkShortcut           *shortcut)
@@ -736,7 +741,11 @@ gtk_shortcut_controller_add_shortcut (GtkShortcutController *self,
 
   widget = gtk_event_controller_get_widget (GTK_EVENT_CONTROLLER (self));
   if (widget)
-    update_accel (shortcut, widget, TRUE);
+    {
+      GtkActionMuxer *muxer = _gtk_widget_get_action_muxer (widget, TRUE);
+
+      update_accel (shortcut, muxer, TRUE);
+    }
 
   g_list_store_append (G_LIST_STORE (self->shortcuts), shortcut);
   g_object_unref (shortcut);
@@ -745,7 +754,7 @@ gtk_shortcut_controller_add_shortcut (GtkShortcutController *self,
 /**
  * gtk_shortcut_controller_remove_shortcut:
  * @self: the controller
- * @shortcut: a #GtkShortcut
+ * @shortcut: a `GtkShortcut`
  *
  * Removes @shortcut from the list of shortcuts handled by @self.
  *
@@ -767,7 +776,11 @@ gtk_shortcut_controller_remove_shortcut (GtkShortcutController  *self,
 
   widget = gtk_event_controller_get_widget (GTK_EVENT_CONTROLLER (self));
   if (widget)
-    update_accel (shortcut, widget, FALSE);
+    {
+      GtkActionMuxer *muxer = _gtk_widget_get_action_muxer (widget, FALSE);
+
+      update_accel (shortcut, muxer, FALSE);
+    }
 
   for (i = 0; i < g_list_model_get_n_items (self->shortcuts); i++)
     {
@@ -785,8 +798,8 @@ gtk_shortcut_controller_remove_shortcut (GtkShortcutController  *self,
 }
 
 /**
- * gtk_shortcut_controller_set_scope:
- * @self: a #GtkShortcutController
+ * gtk_shortcut_controller_set_scope: (attributes org.gtk.Method.set_property=scope)
+ * @self: a `GtkShortcutController`
  * @scope: the new scope to use
  *
  * Sets the controller to have the given @scope.
@@ -798,7 +811,7 @@ gtk_shortcut_controller_remove_shortcut (GtkShortcutController  *self,
  *
  * With %GTK_SHORTCUT_SCOPE_LOCAL, shortcuts will only be activated
  * when the widget has focus.
- **/
+ */
 void
 gtk_shortcut_controller_set_scope (GtkShortcutController *self,
                                    GtkShortcutScope       scope)
@@ -824,14 +837,14 @@ gtk_shortcut_controller_set_scope (GtkShortcutController *self,
 }
 
 /**
- * gtk_shortcut_controller_get_scope:
- * @self: a #GtkShortcutController
+ * gtk_shortcut_controller_get_scope: (attributes org.gtk.Method.get_property=scope)
+ * @self: a `GtkShortcutController`
  *
  * Gets the scope for when this controller activates its shortcuts. See
  * gtk_shortcut_controller_set_scope() for details.
  *
  * Returns: the controller's scope
- **/
+ */
 GtkShortcutScope
 gtk_shortcut_controller_get_scope (GtkShortcutController *self)
 {
@@ -841,8 +854,8 @@ gtk_shortcut_controller_get_scope (GtkShortcutController *self)
 }
 
 /**
- * gtk_shortcut_controller_set_mnemonics_modifiers:
- * @self: a #GtkShortcutController
+ * gtk_shortcut_controller_set_mnemonics_modifiers: (attributes org.gtk.MEthod.set_property=mnemonic-modifiers)
+ * @self: a `GtkShortcutController`
  * @modifiers: the new mnemonics_modifiers to use
  *
  * Sets the controller to have the given @mnemonics_modifiers.
@@ -858,7 +871,7 @@ gtk_shortcut_controller_get_scope (GtkShortcutController *self)
  * This value is only relevant for local shortcut controllers. Global and managed
  * shortcut controllers will have their shortcuts activated from other places which
  * have their own modifiers for activating mnemonics.
- **/
+ */
 void
 gtk_shortcut_controller_set_mnemonics_modifiers (GtkShortcutController *self,
                                                  GdkModifierType        modifiers)
@@ -874,14 +887,13 @@ gtk_shortcut_controller_set_mnemonics_modifiers (GtkShortcutController *self,
 }
 
 /**
- * gtk_shortcut_controller_get_mnemonics_modifiers:
- * @self: a #GtkShortcutController
+ * gtk_shortcut_controller_get_mnemonics_modifiers: (attributes org.gtk.Method.get_property=mnemonic-modifiers)
+ * @self: a `GtkShortcutController`
  *
- * Gets the mnemonics modifiers for when this controller activates its shortcuts. See
- * gtk_shortcut_controller_set_mnemonics_modifiers() for details.
+ * Gets the mnemonics modifiers for when this controller activates its shortcuts.
  *
  * Returns: the controller's mnemonics modifiers
- **/
+ */
 GdkModifierType
 gtk_shortcut_controller_get_mnemonics_modifiers (GtkShortcutController *self)
 {
@@ -889,4 +901,3 @@ gtk_shortcut_controller_get_mnemonics_modifiers (GtkShortcutController *self)
 
   return self->mnemonics_modifiers;
 }
-
