@@ -14,12 +14,10 @@ meson setup ${release_build_dir} || exit
 meson dist -C${release_build_dir} || exit
 
 # now build the docs
-meson configure -Dgtk_doc=true ${release_build_dir} || exit
-ninja -C${release_build_dir} gdk4-doc gsk4-doc gtk4-doc || exit
+meson configure ${release_build_dir} -Dintrospection=enabled -Dgtk_doc=true || exit
+ninja -C${release_build_dir} || exit
 
-tar cf ${release_build_dir}/meson-dist/gdk-docs-${version}.tar.xz -C${release_build_dir} docs/reference/gdk
-tar cf ${release_build_dir}/meson-dist/gsk-docs-${version}.tar.xz -C${release_build_dir} docs/reference/gsk
-tar cf ${release_build_dir}/meson-dist/gtk-docs-${version}.tar.xz -C${release_build_dir} docs/reference/gtk
+tar cf ${release_build_dir}/meson-dist/gtk-docs-${version}.tar.xz -C${release_build_dir} docs/reference
 
 echo -e "\n\nGTK ${version} release on branch ${branch} in ./${release_build_dir}/:\n"
 
